/*----------------------------------------------------------------------------

	Capture header file

--------------------------------------------------------------------------

	Revision History.

----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------

	Enum definitions.

----------------------------------------------------------------------------*/
enum CAM_CLIENT_TYPES
{
	CAM_CLIENT_USER_TYPE=0,
	CAM_CLIENT_WDM_TYPE=1
};

enum CAPTURE_COORDINATE_TYPES
{
	CAPTURE_COORDINATE_CAM=0,
	CAPTURE_COORDINATE_WINDOW=1
};

enum CAPTURE_CALLBACK_TYPES
{
	CAPTURE_CALLBACK_CAPTURE_OCCURRED=1,
	CAPTURE_CALLBACK_CAPTURE_ERROR=2
};

enum CAPTURE_IMAGE_ADUSTMENT_TYPES
{
	CAPTURE_IMAGE_ADJUSTMENT_PER_COLOUR = 0,
	CAPTURE_IMAGE_ADJUSTMENT_COMBINED_COLOURS = 1
};

enum CAM_FORMAT_TYPES
{
	CAM_RAW=0,
	CAM_RGB32=1,
	CAM_RGB24=2,
	CAM_RGBBYTE=20,
	CAM_RGBDOUBLE=21,
	CAM_GREYBYTE=22
};

enum CAPTURE_AUTO_EXPOSURE_MODES
{
	CAPTURE_AUTO_EXPOSURE_OFF=0,
	CAPTURE_AUTO_EXPOSURE_GAIN=1,
	CAPTURE_AUTO_EXPOSURE_INTEGRATION_TIME=2,
	CAPTURE_AUTO_EXPOSURE_GAIN_AND_INTEGRATION_RATIO=3,
	CAPTURE_AUTO_EXPOSURE_GAIN_AND_INTEGRATION_MIN_TO_MAX=4
};

enum CAPTURE_AUTO_ADJUST_MODES
{
	CAPTURE_AUTO_ADJUST_OFF = 0,
	CAPTURE_AUTO_ADJUST_ON = 1
};

enum CAPTURE_OVERLAY_TYPES
{
	CAPTURE_OVERLAY_OFF=0,
	CAPTURE_OVERLAY_STANDARD=1,
	CAPTURE_OVERLAY_RAW_HISTOGRAM=2,
	CAPTURE_OVERLAY_ADJUSTED_HISTOGRAM=3
};

enum CAPTURE_CURSOR_TYPES
{
	CAPTURE_CURSOR_OFF=0,
	CAPTURE_CURSOR_CROSS_HAIRS=1,
	CAPTURE_CURSOR_CROSS_HAIRS_DASHED=2
};

enum COLOUR_CORRECTION_TYPES
{
	COLOUR_CORRECTION_OFF=0,
	COLOUR_CORRECTION_ON=1
};
/*----------------------------------------------------------------------------

	Definitions.

----------------------------------------------------------------------------*/
#define MAX_CAPTURE_HISTOGRAM_BINS 256

/*----------------------------------------------------------------------------

	Structure definitions.

----------------------------------------------------------------------------*/
typedef struct 
{
	unsigned long id;
	unsigned long max_capture_size;
	unsigned char descr[MAX_CAMERA_DESCRIPTION];
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAM_IDENTIFICATION;

typedef struct
{
	double contrast;
	double brightness;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_COLOUR_ADJUSTMENT;

typedef struct
{
	CAPTURE_COLOUR_ADJUSTMENT rgbColourAdjust;
	CAPTURE_COLOUR_ADJUSTMENT redColourAdjust;
	CAPTURE_COLOUR_ADJUSTMENT greenColourAdjust;
	CAPTURE_COLOUR_ADJUSTMENT blueColourAdjust;
	unsigned long greyscale;
	unsigned long pad1;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_IMAGE_ADJUSTMENT;

typedef struct
{
	long cx;
	long cy;
	long dx;
	long dy;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAM_WINDOW;

typedef struct
{
	CAPTURE_IMAGE_ADJUSTMENT image_adjustment;
	CAM_WINDOW cam;
	double gain;
	double integration_time;
	double rate;
	enum CAM_OPERATION_MODES cam_mode;
	enum CAM_CAPTURE_TYPES capture_type;
	enum CAM_FORMAT_TYPES cam_format;
	long res;
	long frame_size;
	long pad1;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_REQUIREMENT_PARAMS;

typedef struct
{
	double flicker_frequency;
	double nominal_gain;
	double min_gain;
	double max_gain;
	double nominal_integration_time;
	double min_integration_time;
	double max_integration_time;
	double gain_and_integration_time_ratio;
	enum CAPTURE_AUTO_EXPOSURE_MODES exposure_mode;
	long num_filter_frames;
	unsigned long pad1;
	unsigned long pad2;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_AUTO_EXPOSURE_PARAMS;

typedef struct
{
	CAPTURE_AUTO_EXPOSURE_PARAMS exposure;
	enum CAPTURE_AUTO_ADJUST_MODES adjust_mode;
	enum CAPTURE_IMAGE_ADUSTMENT_TYPES adjustment_type;
	double min_threshold;
	double max_threshold;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_AUTO_ADJUST_PARAMS;

typedef struct
{
	HWND hwnd;
	long ox;
	long oy;
	long dx;
	long dy;
	unsigned long pad1;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} DEST_WINDOW;

typedef struct
{
	long total;
	long sum;
	long min;
	long max;
	long mean;
	long clipped;
	unsigned long pad2;
	unsigned long pad3;
	unsigned long bin[MAX_CAPTURE_HISTOGRAM_BINS];
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_HISTOGRAM;

typedef struct
{
	CAPTURE_REQUIREMENT_PARAMS params;
	CAPTURE_AUTO_ADJUST_PARAMS auto_adjust;
	CAPTURE_HISTOGRAM rgbHist;
	CAPTURE_HISTOGRAM redHist;
	CAPTURE_HISTOGRAM greenHist;
	CAPTURE_HISTOGRAM blueHist;
	double focus;
	unsigned long frame_number;
	unsigned long time_stamp;
	long num_missed_pixels;
	long pad1;
	double frame_quality;
	unsigned short sync_valid;
	unsigned short spare3;
	unsigned short crc_errors;
	unsigned short usb_errors;
	double actual_rate;
	unsigned long us_time_stamp;
	unsigned long spare8;
} CAPTURE_FRAME_INFO;

typedef struct
{
	enum CAM_SYNC_TYPES sync_type;
	unsigned long invert_logic;
	unsigned long capture;
	unsigned long pad1;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_SYNC_PARAMETERS;

typedef struct
{
	unsigned long display_short_frames;
	unsigned long pad1;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_RENDER_CONFIG;

typedef struct
{
	enum CAPTURE_CALLBACK_TYPES callback_type;
	unsigned long param;
	unsigned long spare1;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_CALLBACK_MSG;

typedef struct
{
	CAPTURE_REQUIREMENT_PARAMS params;
	CAPTURE_AUTO_ADJUST_PARAMS auto_adjust;
	CAPTURE_FRAME_INFO frame_info;
	CAPTURE_HISTOGRAM RawRGBHist;
	CAPTURE_HISTOGRAM RawRedHist;
	CAPTURE_HISTOGRAM RawGreenHist;
	CAPTURE_HISTOGRAM RawBlueHist;
	unsigned long id;
	unsigned short device_index;
	unsigned short capture_index;
	unsigned short requirement_index;
	unsigned short first_auto_exposure;
	CAPTURE_FILTER_QUEUE rgbfilter;
	enum CAPTURE_CONTROL_TYPES control_type;
	unsigned long spare2;
	unsigned long spare3;
	unsigned long spare4;
	unsigned long spare5;
	unsigned long spare6;
	unsigned long spare7;
	unsigned long spare8;
} CAPTURE_REQUIREMENT_ITEM;

typedef struct
{
	unsigned short gain;
	short offset;
} GAIN_OFFSET_COLOUR_STRUCT;

typedef struct
{
	GAIN_OFFSET_COLOUR_STRUCT red;
	GAIN_OFFSET_COLOUR_STRUCT green;
	GAIN_OFFSET_COLOUR_STRUCT blue;
} GAIN_OFFSET_STRUCT;

typedef struct
{
	double redcomp;
	double greencomp;
	double bluecomp;
} COLOUR_WEIGHT;

typedef struct
{
	double pixelscale;
	COLOUR_WEIGHT pixelred;
	COLOUR_WEIGHT pixelgreen;
	COLOUR_WEIGHT pixelblue;
} COLOUR_MATRIX;

typedef struct
{
	enum COLOUR_CORRECTION_TYPES correction_type;
	unsigned long spare1;
	COLOUR_MATRIX matrix;
} COLOUR_CORRECTION_PARAMS;
typedef VOID (_stdcall * CAPTURE_CALLBACK_FUNC) (unsigned long, unsigned long, CAPTURE_CALLBACK_MSG *  );   

typedef struct
{
	unsigned long camera;
	CAPTURE_CALLBACK_FUNC pCallback;
	CAPTURE_REQUIREMENT_PARAMS requirement;
	CAPTURE_AUTO_ADJUST_PARAMS auto_adjust;
} CAPTURE_REQUIREMENT_DEF;

typedef struct
{
	unsigned char description[MAX_CAMERA_DESCRIPTION];
	unsigned long vertical_flip;
	unsigned long spare11;
	unsigned long spare10;
	unsigned long spare9;
	unsigned long spare8;
	unsigned long spare7;
	unsigned long spare6;
	unsigned long spare5;
	unsigned long spare4;
	unsigned long spare3;
	unsigned long spare2;
	unsigned long spare1;
} CAMERA_SETTINGS;

/*----------------------------------------------------------------------------

	Function definitions.

----------------------------------------------------------------------------*/
EXT_DEF char * FN_DEF cam_lib_version(void);

EXT_DEF unsigned long FN_DEF open_cam(enum CAM_CLIENT_TYPES client_type);
EXT_DEF void FN_DEF close_cam(void);
EXT_DEF void FN_DEF reset_cam(void);

EXT_DEF unsigned long FN_DEF num_cameras(void);
EXT_DEF unsigned long FN_DEF alloc_capture(unsigned long camera);
EXT_DEF unsigned long FN_DEF free_capture(unsigned long capture);
EXT_DEF unsigned long FN_DEF chain_capture(unsigned long capture, unsigned long dest);
EXT_DEF unsigned long FN_DEF capture_control(unsigned long capture, enum CAPTURE_CONTROL_TYPES control_type);

EXT_DEF unsigned long FN_DEF setup_capture(unsigned long capture, enum CAM_OPERATION_MODES cam_mode, double rate, CAM_WINDOW *cam, long res, double integration_time, double gain, enum CAM_CAPTURE_TYPES capture_type, enum CAM_FORMAT_TYPES cam_format, long *frame_size, CAPTURE_IMAGE_ADJUSTMENT *adj);
EXT_DEF unsigned long FN_DEF setup_capture_ex(unsigned long capture, enum CAM_OPERATION_MODES cam_mode, double rate, CAM_WINDOW *cam, long res, double integration_time, double gain, enum CAM_CAPTURE_TYPES capture_type, enum CAM_FORMAT_TYPES cam_format, long *frame_size, CAPTURE_IMAGE_ADJUSTMENT *adj);
EXT_DEF unsigned long FN_DEF setup_timer_capture(unsigned long capture, double rate, DEST_WINDOW *dest, enum CAM_OPERATION_MODES cam_mode, CAM_WINDOW *cam, long res, double integration_time, double gain, CAPTURE_IMAGE_ADJUSTMENT *adj);
EXT_DEF unsigned long FN_DEF reset_capture(unsigned long capture);
EXT_DEF unsigned long FN_DEF start_capture_render(unsigned long capture, DEST_WINDOW *dest);
EXT_DEF unsigned long FN_DEF stop_capture_render(unsigned long capture);
EXT_DEF unsigned long FN_DEF capture_occured(unsigned long capture, CAPTURE_FRAME_INFO *info);
EXT_DEF unsigned long FN_DEF capture_overlay(unsigned long capture, enum CAPTURE_COORDINATE_TYPES coordinate_type, enum CAPTURE_OVERLAY_TYPES overlay_type, HBITMAP bitmap_handle, long cx, long cy, unsigned long transparent_colour);
EXT_DEF unsigned long FN_DEF capture_cursor(unsigned long capture, enum CAPTURE_COORDINATE_TYPES coordinate_type, enum CAPTURE_CURSOR_TYPES cursor_type, long cx, long cy, unsigned long cursor_colour, double angle);
EXT_DEF unsigned long FN_DEF force_capture(unsigned long capture);
EXT_DEF unsigned long FN_DEF get_capture_params(unsigned long capture, CAPTURE_REQUIREMENT_PARAMS *params);
EXT_DEF unsigned long FN_DEF get_frame_info(unsigned long capture, CAPTURE_FRAME_INFO *info);
EXT_DEF unsigned long FN_DEF hook_capture_callback(unsigned long capture, unsigned long context, CAPTURE_CALLBACK_FUNC pCallback );

EXT_DEF unsigned long FN_DEF copy_capture_buffer(unsigned long capture, unsigned char *buffer, CAPTURE_FRAME_INFO *info);
EXT_DEF unsigned long FN_DEF flush_capture_buffer(unsigned long capture);

EXT_DEF void FN_DEF set_register(unsigned long camera, unsigned char reg,unsigned short data); 
EXT_DEF unsigned long FN_DEF enumerate_cameras(CAM_IDENTIFICATION *ident, unsigned long num_idents);
EXT_DEF unsigned long FN_DEF get_camera_id(char *descr);
EXT_DEF unsigned long FN_DEF get_camera_description(unsigned long camera, char *descr, unsigned long descr_len);
EXT_DEF unsigned long FN_DEF set_camera_description(unsigned long camera, char *descr);
EXT_DEF unsigned long FN_DEF get_capture_camera_id(unsigned long capture);

EXT_DEF unsigned long FN_DEF get_camera_settings(unsigned long camera, CAMERA_SETTINGS *settings);
EXT_DEF unsigned long FN_DEF set_camera_settings(unsigned long camera, CAMERA_SETTINGS *settings);

EXT_DEF unsigned long FN_DEF setup_auto_adjust(unsigned long capture, CAPTURE_AUTO_ADJUST_PARAMS *params);
EXT_DEF unsigned long FN_DEF get_auto_adjust_params(unsigned long capture, CAPTURE_AUTO_ADJUST_PARAMS *params);
EXT_DEF unsigned long FN_DEF setup_sync(unsigned long camera, unsigned long sync, CAPTURE_SYNC_PARAMETERS *params );

EXT_DEF unsigned long FN_DEF set_capture_render_config(unsigned long capture, CAPTURE_RENDER_CONFIG *config);
EXT_DEF unsigned long FN_DEF get_capture_render_config(unsigned long capture, CAPTURE_RENDER_CONFIG *config);

EXT_DEF unsigned long FN_DEF read_current_capture_user_data(unsigned long capture, void *dest, unsigned long sz);
EXT_DEF unsigned long FN_DEF write_current_capture_user_data(unsigned long capture, void *src, unsigned long sz);

/*----------------------------------------------------------------------------

	Local function prototypes

----------------------------------------------------------------------------*/
EXT_DEF unsigned long FN_DEF capture_occured_ex(unsigned long capture, CAPTURE_FRAME_INFO *info);
EXT_DEF unsigned long FN_DEF copy_capture_buffer_ex(unsigned long capture, unsigned char *buffer, CAPTURE_FRAME_INFO *info);
EXT_DEF unsigned long FN_DEF open_cam_ex(enum CAM_CLIENT_TYPES client_type);
EXT_DEF unsigned long FN_DEF alloc_capture_ex(unsigned long camera);
EXT_DEF unsigned long FN_DEF free_capture_ex(unsigned long capture);
EXT_DEF void FN_DEF close_cam_ex(void);
EXT_DEF void FN_DEF reset_cam_ex(void);
EXT_DEF unsigned short FN_DEF cam_read_eeprom(unsigned long camera, unsigned short addr, unsigned char *buffer, unsigned short len);
EXT_DEF unsigned short FN_DEF cam_write_eeprom(unsigned long camera, unsigned short addr, unsigned char *buffer, unsigned short len);
EXT_DEF unsigned long FN_DEF capture_hdr_to_info(CAPTURE_FRAME_HEADER *hdr, CAPTURE_FRAME_TERMINATOR *term, CAPTURE_FRAME_INFO *info);
EXT_DEF unsigned long FN_DEF capture_params_to_hdr(unsigned short capture_index, CAPTURE_REQUIREMENT_PARAMS *params, CAPTURE_FRAME_HEADER *hdr);
EXT_DEF unsigned long FN_DEF reset_capture_ex(unsigned long capture);
EXT_DEF CAPTURE_REQUIREMENT_ITEM * FN_DEF get_capture_item(unsigned long capture);
EXT_DEF unsigned long FN_DEF capture_control_ex(unsigned long capture, enum CAPTURE_CONTROL_TYPES control_type);
EXT_DEF unsigned long FN_DEF free_frame_capture_buffer_ex(unsigned long capture);
EXT_DEF unsigned long FN_DEF free_frame_capture_buffer(unsigned long capture);
EXT_DEF unsigned long FN_DEF chain_frame_capture_buffer(unsigned long capture);

EXT_DEF unsigned long FN_DEF get_test_string(unsigned long camera, unsigned char *str, unsigned long sz);
EXT_DEF unsigned long FN_DEF set_operating_mode(unsigned long camera, unsigned long operating_mode);

double flicker_adjusted(double integration_time, double freq);
double gain_adjusted(double gain);
double gain_step(double gain);
double calc_next_gain(short target_pixel, short max_pixel, double gain, double scale);
double calc_next_integration_time(short target_pixel, short max_pixel, double integration_time, double scale);
unsigned long init_capture_filter(double value, unsigned short len, CAPTURE_FILTER_QUEUE PTR_DEF *queue);
double capture_filter_value(double value,CAPTURE_FILTER_QUEUE PTR_DEF *queue);
void * __cdecl malloc_mem( size_t size );
void __cdecl free_mem(void *ptr);
double Focus3x3(unsigned char *buffer, CAPTURE_FRAME_HEADER *hdr);

void MapToScaleAndOffset(double black_level, CAPTURE_IMAGE_ADJUSTMENT *adj, GAIN_OFFSET_STRUCT *gainoffset);
void MapFromScaleAndOffset(double black_level, CAPTURE_IMAGE_ADJUSTMENT *adj, GAIN_OFFSET_STRUCT *gainoffset);

