#pragma once

using namespace System;
using namespace System::Collections::Generic;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Reflection;


namespace CPlusPlusSample {

	/// <summary>
	/// Summary for LightingForm
	///
	/// WARNING: If you change the name of this class, you will need to change the
	///          'Resource File Name' property for the managed resource compiler tool
	///          associated with all .resx files this class depends on.  Otherwise,
	///          the designers will not be able to interact properly with localized
	///          resources associated with this form.
	/// </summary>
	public ref class LightingForm : public System::Windows::Forms::Form
	{
        ref class GroupBoxWithoutBackgroundRepaint : GroupBox
        {
			public: GroupBoxWithoutBackgroundRepaint()
            {
				this->SetStyle(ControlStyles::UserPaint, true);
				this->SetStyle(ControlStyles::OptimizedDoubleBuffer, true);
				this->SetStyle(ControlStyles::AllPaintingInWmPaint, true);
                this->UpdateStyles();
            }
        };

		ref class LED
        {
			private:
			int _x, _y;
            bool _on;
			Brush ^_onColor;

            /// <summary>
            /// Prevents a default instance of the <see cref="LED"/> class from being created.
            /// </summary>
			private: LED()
			{
				 _x = 0;
				 _y = 0;
				 _on = false;
				 _onColor = Brushes::White;
			}
            /// <summary>
            /// Initializes a new instance of the <see cref="LED"/> class.
            /// </summary>
            /// <param name="x">The x.</param>
            /// <param name="y">The y.</param>
            /// <param name="on">if set to <c>true</c> [on].</param>
            /// <param name="onColor">Color of the on.</param>
            /// <param name="offColor">Color of the off.</param>
			public: LED(int x, int y, bool on, Brush ^onColor)
            {
                _x = x;
                _y = y;
                _on = on;
                _onColor = onColor;
            }
            /// <summary>
            /// Initializes a new instance of the <see cref="LED"/> class.
            /// </summary>
            /// <param name="x">The x.</param>
            /// <param name="y">The y.</param>
            /// <param name="on">if set to <c>true</c> [on].</param>
            /// <param name="onColor">Color of the on.</param>
			public: LED(double x, double y, bool on, Brush ^onColor)
            {
                _x = (int)x;
                _y = (int)y;
                _on = on;
                _onColor = onColor;
            }

            /// <summary>
            /// Gets or sets a value indicating whether this <see cref="LED"/> is enable.
            /// </summary>
            /// <value>
            ///   <c>true</c> if enable; otherwise, <c>false</c>.
            /// </value>
			public: void EnableLED(bool value)
            {
                _on = value;
            }

			public: bool EnableLED(void)
            {
				return _on;
			}
            /// <summary>
            /// Draws this instance to the specified Graphics object
            /// </summary>
            /// <param name="gfx">The GFX object.</param>
			public: void Draw(Graphics ^gfx)
            {
                if (_on)
                {
                    gfx->FillEllipse(_onColor, _x, _y, 15, 15);
					gfx->DrawEllipse(Pens::DarkGray, _x, _y, 15, 15);
                }
				else
				{
                    gfx->FillEllipse(Brushes::Gray, _x, _y, 15, 15);
					gfx->DrawEllipse(Pens::Gray, _x, _y, 15, 15);
				}
            }

            /// <summary>
            /// Gets or sets the on brush.
            /// </summary>
            /// <value>
            /// The on brush.
            /// </value>
			public: Brush^ DrawColor(void)
            {
                return _onColor;
            }

			public: void DrawColor(Brush ^value)
            {
                _onColor = value;
            }
        };

	private:
		System::Collections::Generic::List<LED ^> ^m_Leds;
        int m_nNumUnits;
        int m_currentLEDIntensity;
        int m_maxLEDIntensity;
        int m_currentDirection;
        int m_maxDirection;
        int m_currentWidth;
        int m_maxWidth;
        bool m_isPowerOn;
        bool m_isMasterOn;

        Image ^m_bufferImage;

        bool m_InInit;
        bool m_is022;
        bool m_is036;
		bool m_hasStepperControl;
		private: System::Windows::Forms::Button ^m_LightButton;

	public:
		LightingForm(System::Windows::Forms::Button ^LightButton)
		{
			short arg1 = 0;
			short arg2 = 0;
			array<Object ^> ^Params = gcnew array<Object ^> (2);
			InitializeComponent();
            m_LightButton = LightButton;
			m_Leds = gcnew System::Collections::Generic::List<LED ^>();
			m_currentLEDIntensity = 255;
			m_maxLEDIntensity = 255;
			m_currentDirection = 0;
			m_maxDirection = 16;
			m_currentWidth = 0;
			m_maxWidth = 16;
			m_isPowerOn = false;
			m_isMasterOn = false;

            m_bufferImage = gcnew Bitmap(this->Width, this->Height);
			Graphics ^gfx = Graphics::FromImage(m_bufferImage);
            {
				gfx->FillEllipse(Brushes::Gray, 20, 20, 220, 220);			//outer circle
				gfx->FillEllipse(SystemBrushes::Control, 80, 80, 100, 100);	//inner circle
			}
			m_is022 = true;
			m_is036 = false;
			m_hasStepperControl = false;
            m_InInit = true;
			this->SetStyle(ControlStyles::UserPaint, true);
			this->SetStyle(ControlStyles::OptimizedDoubleBuffer, true);
			this->SetStyle(ControlStyles::AllPaintingInWmPaint, true);
            this->UpdateStyles();
			int unitType = Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::UNIT_TYPE,0);
			if (unitType == 1)
			{
				//we've got a 022
				m_is022 = true;
				//perform check for number of steppers
				int numSteppers = Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::NUM_STEPPERS,0);
				if (numSteppers > 0)
					this->m_hasStepperControl = true;
			}
			else if(unitType == 2)
			{
				//we'ev got a 026
				this->m_is022 = false;
				this->m_hasStepperControl = true;
			}
            else if (unitType == 3)
            {
                //we'ev got a 036
                this->m_is022 = false;
                this->m_is036 = true;
                this->m_hasStepperControl = false;
            }
            if (this->m_is022)
            {
                BuildRingControl(64);
            }
            else
            {
                if (this->m_is036)
                {
                    //we'll show the aux2 panel + 'control' panel
					int nWidth = Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::RING_WIDTH_MAX,0);
                    BuildRingControl((short)nWidth);

                }
                else
                {
                    BuildRingControl(16);
                }
            }

            if (this->m_hasStepperControl)
            {
            }
            else
            {
            }
            m_InInit = false;
			m_nNumUnits = Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::NUM_UNITS,0);
			m_isMasterOn = (Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::ENABLE, 0) == 0 ? false : true);
			m_isPowerOn = (Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::RING_ENABLE, 0) == 0 ? false : true);
            MasterEnable(m_isMasterOn);
            RingEnable(m_isPowerOn);
			RingDirection((int)Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::RING_DIRECTION, 0));
			RingPower((int)Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::RING_POWER, 0));
			RingWidth((int)Deva::Lghtlib::read_light((short)Deva::Lghtlib::COMMANDS::RING_WIDTH, 0));
		}

		public: void BuildRingControl(int numLEDs)
        {
            switch (numLEDs)
            {
                case 64:
                    m_maxWidth = 32;
                    m_currentWidth = 0;
                    m_maxLEDIntensity = 255;
                    m_currentLEDIntensity = 127;
                    m_maxDirection = 31;
                    m_currentDirection = 0;
                    m_isPowerOn = false;
                    m_isMasterOn = false;
                    BuildExtendedLightring();
                    break;
                case 32:
                    m_maxWidth = 32;
                    m_currentWidth = 0;
                    m_maxLEDIntensity = 255;
                    m_currentLEDIntensity = 127;
                    m_maxDirection = 31;
                    m_currentDirection = 0;
                    m_isPowerOn = false;
                    m_isMasterOn = false;
                    Build036Lightring();
                    break;
                case 16:
                    m_maxWidth = 16;
                    m_currentWidth = 0;
                    m_maxLEDIntensity = 255;
                    m_currentLEDIntensity = 127;
                    m_maxDirection = 15;
                    m_currentDirection = 0;
                    m_isPowerOn = false;
                    m_isMasterOn = false;
                    BuildBasicLightring();
                    break;
            }

            //update width slider & settings
            int tempWidthVal = this->WidthVScrollBar->Value;
            this->WidthVScrollBar->Value = this->WidthVScrollBar->Maximum;
            this->WidthVScrollBar->Minimum = m_maxWidth * -1;
            if (tempWidthVal >= this->WidthVScrollBar->Minimum)
                this->WidthVScrollBar->Value = tempWidthVal;

            //update direction slider & settings
            int tempDirVal = this->DirectionHScrollBar->Value;
            this->DirectionHScrollBar->Value = DirectionHScrollBar->Minimum;
            this->DirectionHScrollBar->Maximum = m_maxDirection;
            if (tempDirVal <= this->DirectionHScrollBar->Maximum)
                this->DirectionHScrollBar->Value = tempDirVal;

            this->PowerVScrollBar->Minimum = -255;
        }

		private: void BuildBasicLightring(void)
        {
            //new lightring count = 16LEDs
            if (m_Leds->Count != 16)
            {
                //clear the current led list
                m_Leds->Clear();
                //calculate new draw arrangement

                int rad = 80;
                int toppos = 122;
                int leftpos = 123;
                for (int index = 0; index < 16; index++)
                {
					m_Leds->Add(gcnew LED(leftpos + rad * Math::Cos((15.5 - index) / 8 * Math::PI),
						toppos - rad * Math::Sin((15.5 - index) / 8 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity()));
                }
            }
            this->LightRingGroupBox->Invalidate();
        }

		private: void BuildExtendedLightring()
        {
            //new lightring count = 64 LEDs
            if (m_Leds->Count != 64)
            {
                //clear the current led list
                m_Leds->Clear();
                //calculate new draw arrangement

                int innerRadius = 65;
                int innerAltRadius = 85;
                int outerRadius = 75;
                int outerAltRadius = 95;

                int toppos = 122;
                int leftpos = 123;
				array<LED ^> ^ledArray = gcnew array<LED ^> (64);

                for (int index = 0; index < 64; index += 4)
                {
					ledArray[index] = gcnew LED(leftpos + innerRadius * Math::Cos((index) / 32.0 * Math::PI),
						toppos - innerRadius * Math::Sin((index) / 32.0 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity());
                    //}
                    //radius = 85;
                    //for(int index=0; index < 64; index+=4)
                    //{
					ledArray[index + 1] = gcnew LED(leftpos + innerAltRadius * Math::Cos((index) / 32.0 * Math::PI),
						toppos - innerAltRadius * Math::Sin((index) / 32.0 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity());
                    //}
                    //radius = 75;
                    //for(int index=0; index < 64; index+=4)
                    //{
					ledArray[index + 2] = gcnew LED(leftpos + outerRadius * Math::Cos((index + 2) / 32.0 * Math::PI),
						toppos - outerRadius * Math::Sin((index + 2) / 32.0 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity());
                    //}
                    //radius = 95;
                    //for(int index=0; index < 64; index+=4)
                    //{
					ledArray[index + 3] = gcnew LED(leftpos + outerAltRadius * Math::Cos((index + 2) / 32.0 * Math::PI),
						toppos - outerAltRadius * Math::Sin((index + 2) / 32.0 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity());
                }
				for (int index = 0;index < 64;index++)
				{
	                this->m_Leds->Add(ledArray[index]);
				}
                //force this control to re-draw at the next cycle
                this->LightRingGroupBox->Invalidate();
            }
        }

		private: void Build036Lightring()
        {
            //new lightring count = 32 LEDs
            if (m_Leds->Count != 32)
            {
                //clear the current led list
                m_Leds->Clear();
                //calculate new draw arrangement

                int innerRadius = 75;
                int outerRadius = 90;

                int toppos = 122;
                int leftpos = 123;
                array<LED ^> ^ledArray = gcnew array<LED ^> (32);

                for (int index = 0; index < 32; index += 2)
                {
					ledArray[index] = gcnew LED(leftpos + innerRadius * Math::Cos((index) / 16.0 * Math::PI),
						toppos - innerRadius * Math::Sin((index) / 16.0 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity());

					ledArray[index + 1] = gcnew LED(leftpos + outerRadius * Math::Cos((index + 1) / 16.0 * Math::PI),
						toppos - outerRadius * Math::Sin((index + 1) / 16.0 * Math::PI),
                        m_isPowerOn, GetBrushFromIntensity());
                }
				for (int index = 0;index < 32;index++)
				{
	                this->m_Leds->Add(ledArray[index]);
				}

                //force this control to re-draw at the next cycle
                this->LightRingGroupBox->Invalidate();
            }
        }

		public: bool MasterEnable(void)
        {
	        return m_isMasterOn;
        }

		public: void MasterEnable(bool value)
        {
            m_isMasterOn = value;
            UpdateLEDsForNewSettings();
        }

		public: bool RingEnable(void)
        {
            return m_isPowerOn;
        }

		public: void RingEnable(bool value)
        {
            m_isPowerOn = value;
            EnableCheckBox->Checked = (m_isPowerOn && m_isMasterOn);
            UpdateLEDsForNewSettings();
        }

		public: int RingPower(void)
        {
            return m_currentLEDIntensity;
        }

		public: void RingPower(int value)
		{
            int tmp = value;

            if (tmp > m_maxLEDIntensity)
                tmp = m_maxLEDIntensity;

			this->m_currentLEDIntensity = tmp;

            this->PowerVScrollBar->Value = (-1 * tmp);
            this->PowerTextBox->Text = tmp.ToString();
            UpdateLEDsForNewSettings();
        }

		public: int RingDirection(void)
        {
            return m_currentDirection;
        }

		public: void RingDirection(int value)
        {
            int tmp = value;
            if (tmp > this->m_maxDirection)
                tmp = m_maxDirection;

            m_currentDirection = tmp;
            this->DirectionHScrollBar->Value = tmp;
            this->DirectionTextBox->Text = tmp.ToString();
            UpdateLEDsForNewSettings();
        }

		public: int RingWidth(void)
        {
            return m_currentWidth;
        }

		public: void RingWidth(int value)
        {
            int tmp = value;
            if (tmp > this->m_maxWidth)
                tmp = this->m_maxWidth;

            this->m_currentWidth = tmp;

            this->WidthVScrollBar->Value = (-1 * tmp);
            this->WidthTextBox->Text = tmp.ToString();
            UpdateLEDsForNewSettings();
        }

		private: Brush ^GetBrushFromIntensity(void)
        {
            int drawVal = (this->m_currentLEDIntensity / 2) + 127;
			return gcnew SolidBrush(Color::FromArgb(drawVal,
                                                drawVal,
                                                drawVal));
        }

		private: void UpdateLEDsForNewSettings(void)
        {
            Brush ^color = GetBrushFromIntensity();
            int modulusValue = 0;
            int startIndex = 0;

            if (m_InInit)
            {
                return;
            }

            switch (this->m_Leds->Count)
            {
                case 64:
                    modulusValue = 32;
                    startIndex = m_currentDirection - ((m_currentWidth - 1) / 2);
                    break;
                case 32:
                    modulusValue = 32;
                    startIndex = m_currentDirection - ((m_currentWidth - 1) / 2);
                    break;
                case 16:
                    modulusValue = 16;
                    startIndex = -m_currentDirection - ((m_currentWidth - 1) / 2);
                    break;
            }

            int endIndex = startIndex + m_currentWidth - 1;

            startIndex = (startIndex + 32) % modulusValue;
            endIndex = (endIndex + 32) % modulusValue;

            for (int index = 0; index < modulusValue; index++)
            {
                if (((endIndex >= startIndex && index >= startIndex && index <= endIndex) ||
                    (startIndex > endIndex && (index >= startIndex || index <= endIndex))) &&
                    m_currentWidth > 0)
                {
                    switch (this->m_Leds->Count)
                    {
                        case 64:
							{
                            int extendedIndex = index * 2;
                            this->m_Leds[extendedIndex]->DrawColor(color);
                            this->m_Leds[extendedIndex]->EnableLED(this->m_isPowerOn && this->m_isMasterOn);
                            this->m_Leds[extendedIndex + 1]->DrawColor(color);
                            this->m_Leds[extendedIndex + 1]->EnableLED(this->m_isPowerOn && this->m_isMasterOn);
                            break;
							}
                        case 32:
                            this->m_Leds[index]->DrawColor(color);
                            this->m_Leds[index]->EnableLED(this->m_isPowerOn && this->m_isMasterOn);
                            break;
                        case 16:
                            this->m_Leds[index]->DrawColor(color);
                            this->m_Leds[index]->EnableLED(this->m_isPowerOn && this->m_isMasterOn);
                            break;
                    }
                }
                else
                {
                    switch (this->m_Leds->Count)
                    {
                        case 64:
							{
                            int extendedIndex = index * 2;
                            this->m_Leds[extendedIndex]->EnableLED(false);
                            this->m_Leds[extendedIndex + 1]->EnableLED(false);
                            break;
							}
                        case 32:
                            this->m_Leds[index]->EnableLED(false);
                            break;
                        case 16:
                            this->m_Leds[index]->EnableLED(false);
                            break;
                    }
                }
            }


			Graphics ^gfx = Graphics::FromImage(m_bufferImage);
            {
				int nLed;
				for(nLed = 0;nLed < m_Leds->Count;nLed++)
                {
                    m_Leds[nLed]->Draw(gfx);
                }
            }
			this->LightRingGroupBox->Invalidate(gcnew System::Drawing::Region(RectangleF(20, 20, 220, 220)),false);	//invalidate only the lightring section of the image
        }

	protected:
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		~LightingForm()
		{
			if (components)
			{
				delete components;
			}
		}
	private: GroupBoxWithoutBackgroundRepaint^  LightRingGroupBox;
	private: System::Windows::Forms::Button^  CloseButton;
	private: System::Windows::Forms::HScrollBar^  DirectionHScrollBar;
	private: System::Windows::Forms::Label^  DirectionLabel;
	private: System::Windows::Forms::TextBox^  DirectionTextBox;
	private: System::Windows::Forms::CheckBox^  EnableCheckBox;
	private: System::Windows::Forms::TextBox^  WidthTextBox;
	private: System::Windows::Forms::TextBox^  PowerTextBox;
	private: System::Windows::Forms::Label^  PowerLabel;
	private: System::Windows::Forms::Label^  WidthLabel;
	private: System::Windows::Forms::VScrollBar^  WidthVScrollBar;
	private: System::Windows::Forms::VScrollBar^  PowerVScrollBar;

	protected: 
	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container ^components;

#pragma region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->LightRingGroupBox = (gcnew GroupBoxWithoutBackgroundRepaint());
			this->PowerVScrollBar = (gcnew System::Windows::Forms::VScrollBar());
			this->WidthVScrollBar = (gcnew System::Windows::Forms::VScrollBar());
			this->PowerLabel = (gcnew System::Windows::Forms::Label());
			this->WidthLabel = (gcnew System::Windows::Forms::Label());
			this->PowerTextBox = (gcnew System::Windows::Forms::TextBox());
			this->WidthTextBox = (gcnew System::Windows::Forms::TextBox());
			this->EnableCheckBox = (gcnew System::Windows::Forms::CheckBox());
			this->DirectionTextBox = (gcnew System::Windows::Forms::TextBox());
			this->DirectionLabel = (gcnew System::Windows::Forms::Label());
			this->DirectionHScrollBar = (gcnew System::Windows::Forms::HScrollBar());
			this->CloseButton = (gcnew System::Windows::Forms::Button());
			this->LightRingGroupBox->SuspendLayout();
			this->SuspendLayout();
			// 
			// LightRingGroupBox
			// 
			this->LightRingGroupBox->Controls->Add(this->PowerVScrollBar);
			this->LightRingGroupBox->Controls->Add(this->WidthVScrollBar);
			this->LightRingGroupBox->Controls->Add(this->PowerLabel);
			this->LightRingGroupBox->Controls->Add(this->WidthLabel);
			this->LightRingGroupBox->Controls->Add(this->PowerTextBox);
			this->LightRingGroupBox->Controls->Add(this->WidthTextBox);
			this->LightRingGroupBox->Controls->Add(this->EnableCheckBox);
			this->LightRingGroupBox->Controls->Add(this->DirectionTextBox);
			this->LightRingGroupBox->Controls->Add(this->DirectionLabel);
			this->LightRingGroupBox->Controls->Add(this->DirectionHScrollBar);
			this->LightRingGroupBox->Location = System::Drawing::Point(5, 7);
			this->LightRingGroupBox->Name = L"LightRingGroupBox";
			this->LightRingGroupBox->Size = System::Drawing::Size(394, 308);
			this->LightRingGroupBox->TabIndex = 0;
			this->LightRingGroupBox->TabStop = false;
			this->LightRingGroupBox->Text = L"Light Ring";
			this->LightRingGroupBox->Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &LightingForm::OnLightRingGroupBoxPaint);
			// 
			// PowerVScrollBar
			// 
			this->PowerVScrollBar->LargeChange = 1;
			this->PowerVScrollBar->Location = System::Drawing::Point(355, 41);
			this->PowerVScrollBar->Maximum = 0;
			this->PowerVScrollBar->Minimum = -255;
			this->PowerVScrollBar->Name = L"PowerVScrollBar";
			this->PowerVScrollBar->Size = System::Drawing::Size(21, 210);
			this->PowerVScrollBar->TabIndex = 2;
			this->PowerVScrollBar->Value = -255;
			this->PowerVScrollBar->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &LightingForm::OnPowerScroll);
			// 
			// WidthVScrollBar
			// 
			this->WidthVScrollBar->LargeChange = 1;
			this->WidthVScrollBar->Location = System::Drawing::Point(312, 41);
			this->WidthVScrollBar->Maximum = 0;
			this->WidthVScrollBar->Minimum = -31;
			this->WidthVScrollBar->Name = L"WidthVScrollBar";
			this->WidthVScrollBar->Size = System::Drawing::Size(22, 210);
			this->WidthVScrollBar->TabIndex = 1;
			this->WidthVScrollBar->Value = -31;
			this->WidthVScrollBar->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &LightingForm::OnWidthScroll);
			// 
			// PowerLabel
			// 
			this->PowerLabel->AutoSize = true;
			this->PowerLabel->Location = System::Drawing::Point(345, 16);
			this->PowerLabel->Name = L"PowerLabel";
			this->PowerLabel->Size = System::Drawing::Size(37, 13);
			this->PowerLabel->TabIndex = 7;
			this->PowerLabel->Text = L"Power";
			// 
			// WidthLabel
			// 
			this->WidthLabel->AutoSize = true;
			this->WidthLabel->Location = System::Drawing::Point(301, 16);
			this->WidthLabel->Name = L"WidthLabel";
			this->WidthLabel->Size = System::Drawing::Size(35, 13);
			this->WidthLabel->TabIndex = 6;
			this->WidthLabel->Text = L"Width";
			// 
			// PowerTextBox
			// 
			this->PowerTextBox->Location = System::Drawing::Point(355, 254);
			this->PowerTextBox->Name = L"PowerTextBox";
			this->PowerTextBox->ReadOnly = true;
			this->PowerTextBox->Size = System::Drawing::Size(27, 20);
			this->PowerTextBox->TabIndex = 5;
			this->PowerTextBox->TabStop = false;
			this->PowerTextBox->Text = L"0";
			this->PowerTextBox->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			// 
			// WidthTextBox
			// 
			this->WidthTextBox->Location = System::Drawing::Point(311, 254);
			this->WidthTextBox->Name = L"WidthTextBox";
			this->WidthTextBox->ReadOnly = true;
			this->WidthTextBox->Size = System::Drawing::Size(25, 20);
			this->WidthTextBox->TabIndex = 4;
			this->WidthTextBox->TabStop = false;
			this->WidthTextBox->Text = L"0";
			this->WidthTextBox->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			// 
			// EnableCheckBox
			// 
			this->EnableCheckBox->AutoSize = true;
			this->EnableCheckBox->Location = System::Drawing::Point(312, 280);
			this->EnableCheckBox->Name = L"EnableCheckBox";
			this->EnableCheckBox->Size = System::Drawing::Size(59, 17);
			this->EnableCheckBox->TabIndex = 3;
			this->EnableCheckBox->Text = L"Enable";
			this->EnableCheckBox->UseVisualStyleBackColor = true;
			this->EnableCheckBox->CheckedChanged += gcnew System::EventHandler(this, &LightingForm::OnEnableChanged);
			// 
			// DirectionTextBox
			// 
			this->DirectionTextBox->Location = System::Drawing::Point(272, 278);
			this->DirectionTextBox->Name = L"DirectionTextBox";
			this->DirectionTextBox->ReadOnly = true;
			this->DirectionTextBox->Size = System::Drawing::Size(28, 20);
			this->DirectionTextBox->TabIndex = 2;
			this->DirectionTextBox->TabStop = false;
			this->DirectionTextBox->Text = L"0";
			this->DirectionTextBox->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			// 
			// DirectionLabel
			// 
			this->DirectionLabel->AutoSize = true;
			this->DirectionLabel->Location = System::Drawing::Point(12, 285);
			this->DirectionLabel->Name = L"DirectionLabel";
			this->DirectionLabel->Size = System::Drawing::Size(49, 13);
			this->DirectionLabel->TabIndex = 1;
			this->DirectionLabel->Text = L"Direction";
			// 
			// DirectionHScrollBar
			// 
			this->DirectionHScrollBar->LargeChange = 1;
			this->DirectionHScrollBar->Location = System::Drawing::Point(15, 254);
			this->DirectionHScrollBar->Maximum = 31;
			this->DirectionHScrollBar->Name = L"DirectionHScrollBar";
			this->DirectionHScrollBar->Size = System::Drawing::Size(285, 21);
			this->DirectionHScrollBar->TabIndex = 0;
			this->DirectionHScrollBar->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &LightingForm::OnDirectionScroll);
			// 
			// CloseButton
			// 
			this->CloseButton->Location = System::Drawing::Point(427, 292);
			this->CloseButton->Name = L"CloseButton";
			this->CloseButton->Size = System::Drawing::Size(75, 23);
			this->CloseButton->TabIndex = 0;
			this->CloseButton->Text = L"Close";
			this->CloseButton->UseVisualStyleBackColor = true;
			this->CloseButton->Click += gcnew System::EventHandler(this, &LightingForm::OnClose);
			// 
			// LightingForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->ClientSize = System::Drawing::Size(514, 327);
			this->Controls->Add(this->CloseButton);
			this->Controls->Add(this->LightRingGroupBox);
			this->Name = L"LightingForm";
			this->Text = L"Lighting Form";
			this->LightRingGroupBox->ResumeLayout(false);
			this->LightRingGroupBox->PerformLayout();
			this->MaximizeBox = false;
			this->Name = L"LightingForm";
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->FormClosing += gcnew System::Windows::Forms::FormClosingEventHandler(this, &LightingForm::LightingForm_FormClosing);
			this->ResumeLayout(false);
		}
#pragma endregion
	private: System::Void OnLightRingGroupBoxPaint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e) {
            //local double buffering attempt
            if (m_bufferImage != nullptr)
            {
				e->Graphics->DrawImage(m_bufferImage, e->ClipRectangle.X, e->ClipRectangle.Y, e->ClipRectangle, System::Drawing::GraphicsUnit::Pixel);
            }
			 }
	private: System::Void OnClose(System::Object^  sender, System::EventArgs^  e) {
	            m_LightButton->Enabled = true;
				 this->Close();
			 }
	private: System::Void OnWidthScroll(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e) {
            this->m_currentWidth = (-1 * this->WidthVScrollBar->Value);
            this->WidthTextBox->Text = (-1 * this->WidthVScrollBar->Value).ToString();
            if (!m_InInit)
            {
				Deva::Lghtlib::write_light((short)Deva::Lghtlib::COMMANDS::RING_WIDTH,0,this->m_currentWidth);
                UpdateLEDsForNewSettings();
            }
			 }
private: System::Void OnDirectionScroll(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e) {
            this->m_currentDirection = this->DirectionHScrollBar->Value;
            this->DirectionTextBox->Text = this->DirectionHScrollBar->Value.ToString();
            if (!m_InInit)
            {
				Deva::Lghtlib::write_light((short)Deva::Lghtlib::COMMANDS::RING_DIRECTION,0,this->m_currentDirection);
                UpdateLEDsForNewSettings();
            }
		 }
private: System::Void OnPowerScroll(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e) {
            this->m_currentLEDIntensity = -1 * this->PowerVScrollBar->Value;
            this->PowerTextBox->Text = (-1 * this->PowerVScrollBar->Value).ToString();
            if (!m_InInit)
            {
				Deva::Lghtlib::write_light((short)Deva::Lghtlib::COMMANDS::RING_POWER,0,this->m_currentLEDIntensity);
                UpdateLEDsForNewSettings();
            }
		 }
private: System::Void OnEnableChanged(System::Object^  sender, System::EventArgs^  e) {
			short arg1 = 0;
			short arg2 = 0;
			int arg3 = 0;
			array<Object ^> ^Params = gcnew array<Object ^> (3);
            this->m_isPowerOn = this->EnableCheckBox->Checked;
            this->m_isMasterOn = this->m_isPowerOn;
            // update the value on the lightring here
            if (!m_InInit)
            {
				Deva::Lghtlib::write_light((short)Deva::Lghtlib::COMMANDS::ENABLE,0,this->m_isMasterOn != false ? 1 : 0);
				Deva::Lghtlib::write_light((short)Deva::Lghtlib::COMMANDS::RING_ENABLE,0,this->m_isPowerOn != false ? 1 : 0);
                UpdateLEDsForNewSettings();
            }
		 }
private: System::Void LightingForm_FormClosing(System::Object^  sender, System::Windows::Forms::FormClosingEventArgs^  e) {
	            m_LightButton->Enabled = true;
		 }
};
	
}
