using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using Deva;

namespace DevaCameraSample
{
    public partial class DevaCameraSampleApplication : Form
    {
        Deva.Camlib m_Camlib;
        uint m_CamLibHandle;
        UInt32 m_NumCameras;
        Deva.Camlib.CAM_IDENTIFICATION [] m_Idents;
        UInt32 m_CameraId;
        UInt32 m_CaptureId;
        UInt32 m_StillCaptureId;
        UInt32 m_ChainCaptureId;
        bool m_bCapturing;
        bool m_bCapturingStill;
        int m_BuffSize;
        StillPictureForm [] m_Pics;
        int m_nPictureIndex;
        int m_nSkipCount;
        bool m_bUsingLights;
        public DevaCameraSampleApplication()
        {
            InitializeComponent();
            m_CamLibHandle = 0;
            m_bCapturing = false;
            m_bCapturingStill = false;
            m_nSkipCount = 0;
            m_nPictureIndex = 0;
            m_bUsingLights = false;
        }

        private void OnLoad(object sender, EventArgs e)
        {
            m_Camlib = new Camlib();
            m_CamLibHandle = Deva.Camlib.open_cam(Camlib.CAM_CLIENT_TYPES.CAM_CLIENT_USER_TYPE);
            if (m_CamLibHandle == 0)
            {
                MessageBox.Show("Error on open_cam");
                Close();
                return;
            }
            Deva.Camlib.reset_cam();       // reset the camera drivers
            m_NumCameras = Deva.Camlib.num_cameras();
            if (m_NumCameras == 0)      // no cameras so we have a problem
            {
                MessageBox.Show("Error no Cameras");
                Deva.Camlib.close_cam();
                Close();
                return;
            }
            m_Idents = new Deva.Camlib.CAM_IDENTIFICATION [m_NumCameras];
            m_NumCameras = Deva.Camlib.enumerate_cameras(m_Idents, m_NumCameras);
            if (m_NumCameras > 1)       // we have more then 1 camera so select one
            {
                CameraSelectForm Select = new CameraSelectForm(m_Idents, m_NumCameras);
                Select.ShowDialog();
                m_CameraId = (UInt32)m_Idents[Select.GetSelection()].id;
            }
            else
            {
                m_CameraId = (UInt32)m_Idents[0].id;
            }
            this.CaptureButton.Enabled = false;
            this.SaveImageButton.Enabled = false;
            this.VideoButton.Enabled = true;


            short theResult = Deva.Lghtlib.open_light();
            if (theResult != 0)     // error
            {
                m_bUsingLights = false;
            }
            else
            {
                LightingButton.Visible = true;
                LightingButton.Enabled = true;
            }
             //remaining code here
        }

        private void OnClosing(object sender, FormClosingEventArgs e)
        {
            if (m_bCapturing)
            {
                m_bCapturing = false;
                Deva.Camlib.free_capture(m_CaptureId);
                if (m_StillCaptureId != 0)
                {
                    Deva.Camlib.free_capture(m_StillCaptureId);
                }
                if (m_ChainCaptureId != 0)
                {
                    Deva.Camlib.free_capture(m_ChainCaptureId);
                }
            }
            if ( m_CamLibHandle != 0 )
                Deva.Camlib.close_cam();
            if (m_bUsingLights)
            {
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.ENABLE, 0, 0);
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.RING_ENABLE, 0, 0);
                Deva.Lghtlib.close_light();
            }
        }

        private void OnCaptureStill(object sender, EventArgs e)
        {
            if (m_bCapturing && !m_bCapturingStill)
            {
                this.CaptureButton.Enabled = false;     // manually capture a single frame in hi-res
                StillTimer.Enabled = true;
                m_bCapturingStill = true;
                m_nSkipCount = 3;
                Deva.Camlib.capture_control(m_StillCaptureId, Deva.Camlib.CAPTURE_CONTROL_TYPES.CAPTURE_CONTROL_TYPE_FRAME_ADVANCE);
            }
        }

        private void OnClose(object sender, EventArgs e)
        {
            this.Close();
        }

        private void OnCapture(object sender, EventArgs e)
        {
            if (!m_bCapturing)      // start capturing
            {
                m_bCapturing = true;
                this.VideoButton.Text = "Stop";
                this.CaptureButton.Enabled = true;
                this.HistogramCheckBox.Checked = false;
                this.HistogramCheckBox.Enabled = true;
                // Create a timer capture that we leave running this draws itsself into the window area that we specify.
                m_CaptureId = Deva.Camlib.alloc_capture(m_CameraId);
                if (m_CaptureId == 0)       // error no capture
                {
                    MessageBox.Show("Error on alloc_capture");
                    this.VideoButton.Text = "Start";
                    m_bCapturing = false;
                    this.CaptureButton.Enabled = false;
                }
                Deva.Camlib.reset_capture(m_CaptureId);
                Deva.Camlib.DEST_WINDOW DestWnd = new Deva.Camlib.DEST_WINDOW();
                DestWnd.hwnd = this.PicturePanel.Handle;
                DestWnd.dx = this.PicturePanel.Width;
                DestWnd.dy = this.PicturePanel.Height;
                DestWnd.ox = this.PicturePanel.Left;
                DestWnd.oy = this.PicturePanel.Top;
                Deva.Camlib.CAM_OPERATION_MODES OpMode = new Deva.Camlib.CAM_OPERATION_MODES();
                OpMode = Camlib.CAM_OPERATION_MODES.CAM_STANDARD_RES_FULL_SPEED;
                Deva.Camlib.CAM_WINDOW CamWnd = new Deva.Camlib.CAM_WINDOW();
                CamWnd.cx = 0;
                CamWnd.cy = 0;
                CamWnd.dx = 512;
                CamWnd.dy = 384;
                Deva.Camlib.CAPTURE_IMAGE_ADJUSTMENT ImageAdj =  new Deva.Camlib.CAPTURE_IMAGE_ADJUSTMENT();
                ImageAdj.rgbColourAdjust.brightness = 0.0;
                ImageAdj.rgbColourAdjust.contrast = 1.0;
                ImageAdj.redColourAdjust.brightness = 0.0;
                ImageAdj.redColourAdjust.contrast = 1.0;
                ImageAdj.greenColourAdjust.brightness = 0.0;
                ImageAdj.greenColourAdjust.contrast = 1.0;
                ImageAdj.blueColourAdjust.brightness = 0.0;
                ImageAdj.blueColourAdjust.contrast = 1.0;
                if (Deva.Camlib.setup_timer_capture(m_CaptureId, 5.0, ref DestWnd, OpMode, ref CamWnd, 1, 100, 1.0, ref ImageAdj) == 0)
                {
                    MessageBox.Show("Error on setup_timer_capture");
                    this.VideoButton.Text = "Start";
                    m_bCapturing = false;
                    this.CaptureButton.Enabled = false;
                    Deva.Camlib.reset_capture(m_CaptureId);
                    Deva.Camlib.free_capture(m_CaptureId);
                    m_CaptureId = 0;
                }
                // set it up to auto-adjust image quality
                Camlib.CAPTURE_AUTO_ADJUST_PARAMS adjust_params = new Camlib.CAPTURE_AUTO_ADJUST_PARAMS();
                adjust_params.adjust_mode = Camlib.CAPTURE_AUTO_ADJUST_MODES.CAPTURE_AUTO_ADJUST_ON;
                adjust_params.max_threshold = 1.0;
                adjust_params.min_threshold = 1.0;
                adjust_params.adjustment_type = Camlib.CAPTURE_IMAGE_ADUSTMENT_TYPES.CAPTURE_IMAGE_ADJUSTMENT_PER_COLOUR;
                adjust_params.exposure.exposure_mode = Camlib.CAPTURE_AUTO_EXPOSURE_MODES.CAPTURE_AUTO_EXPOSURE_GAIN;
                adjust_params.exposure.nominal_gain = 1;
                adjust_params.exposure.max_gain = 32.0;
                adjust_params.exposure.min_gain = 0.25;
                adjust_params.exposure.max_integration_time = 100;
                adjust_params.exposure.nominal_integration_time = 20;
                adjust_params.exposure.min_integration_time = 1;
                if (Deva.Camlib.setup_auto_adjust(m_CaptureId, ref adjust_params) == 0)
                {
                    MessageBox.Show("Error on setup_auto_adjust");
                    this.VideoButton.Text = "Start";
                    m_bCapturing = false;
                    this.CaptureButton.Enabled = false;
                    Deva.Camlib.reset_capture(m_CaptureId);
                    Deva.Camlib.free_capture(m_CaptureId);
                    m_CaptureId = 0;
                }
                // create a capture for a still picture this will not capture any frames untill we tell it too, and we have to
                // retrieve, process and display the data ourselves. We use a timer to check for available data having requested a frame.
                m_StillCaptureId = Deva.Camlib.alloc_capture(m_CameraId);
                if (m_StillCaptureId == 0)       // error no capture
                {
                    MessageBox.Show("Error on alloc_capture for secondary capture");
                    this.CaptureButton.Enabled = false;
                }
                Deva.Camlib.reset_capture(m_StillCaptureId);
                Deva.Camlib.CAM_OPERATION_MODES StillOpMode = new Deva.Camlib.CAM_OPERATION_MODES();
                StillOpMode = Camlib.CAM_OPERATION_MODES.CAM_STANDARD_RES_FULL_SPEED;
                Deva.Camlib.CAM_WINDOW StillCamWnd = new Deva.Camlib.CAM_WINDOW();
                StillCamWnd.cx = 0;
                StillCamWnd.cy = 0;
                StillCamWnd.dx = 2048;
                StillCamWnd.dy = 1536;
                Deva.Camlib.CAPTURE_IMAGE_ADJUSTMENT StillImageAdj = new Deva.Camlib.CAPTURE_IMAGE_ADJUSTMENT();
                StillImageAdj.rgbColourAdjust.brightness = 0.0;
                StillImageAdj.rgbColourAdjust.contrast = 1.0;
                StillImageAdj.redColourAdjust.brightness = 0.0;
                StillImageAdj.redColourAdjust.contrast = 1.0;
                StillImageAdj.greenColourAdjust.brightness = 0.0;
                StillImageAdj.greenColourAdjust.contrast = 1.0;
                StillImageAdj.blueColourAdjust.brightness = 0.0;
                StillImageAdj.blueColourAdjust.contrast = 1.0;
                if (Deva.Camlib.setup_capture(m_StillCaptureId,StillOpMode, 1.0,ref StillCamWnd, 1, 100, 1.0,
                    Deva.Camlib.CAM_CAPTURE_TYPES.CAPTURE_IMMEDIATE,Deva.Camlib.CAM_FORMAT_TYPES.CAM_RGB32,ref m_BuffSize,
                    ref ImageAdj) == 0)
                {
                    MessageBox.Show("Error on setup_capture");
                    this.CaptureButton.Enabled = false;
                    Deva.Camlib.reset_capture(m_StillCaptureId);
                    Deva.Camlib.free_capture(m_StillCaptureId);
                    m_StillCaptureId = 0;
                }
                Camlib.CAPTURE_AUTO_ADJUST_PARAMS still_adjust_params = new Camlib.CAPTURE_AUTO_ADJUST_PARAMS();
                still_adjust_params.adjust_mode = Camlib.CAPTURE_AUTO_ADJUST_MODES.CAPTURE_AUTO_ADJUST_ON;
                still_adjust_params.max_threshold = 1.0;
                still_adjust_params.min_threshold = 1.0;
                still_adjust_params.adjustment_type = Camlib.CAPTURE_IMAGE_ADUSTMENT_TYPES.CAPTURE_IMAGE_ADJUSTMENT_PER_COLOUR;
                still_adjust_params.exposure.exposure_mode = Camlib.CAPTURE_AUTO_EXPOSURE_MODES.CAPTURE_AUTO_EXPOSURE_GAIN;
                still_adjust_params.exposure.nominal_gain = 1;
                still_adjust_params.exposure.max_gain = 32.0;
                still_adjust_params.exposure.min_gain = 0.25;
                still_adjust_params.exposure.max_integration_time = 100;
                still_adjust_params.exposure.nominal_integration_time = 20;
                still_adjust_params.exposure.min_integration_time = 1;
                if (Deva.Camlib.setup_auto_adjust(m_StillCaptureId, ref still_adjust_params) == 0)
                {
                    MessageBox.Show("Error on secondary setup_auto_adjust");
                    m_bCapturing = false;
                    this.CaptureButton.Enabled = false;
                    Deva.Camlib.reset_capture(m_StillCaptureId);
                    Deva.Camlib.free_capture(m_StillCaptureId);
                    m_StillCaptureId = 0;
                }
                // create a capture that we will chain to the timer capture so that we can monitor the information about frames in the 
                // timer capture and show these in the UI. We use a timer to check for available data and extra the Frame info only.
                m_ChainCaptureId = Deva.Camlib.alloc_capture(m_CameraId);
                if (m_ChainCaptureId == 0)       // error no capture
                {
                    MessageBox.Show("Error on alloc_capture for tertiary capture");
                    this.CaptureButton.Enabled = false;
                }
                if (Deva.Camlib.chain_capture(m_CaptureId, (int)m_ChainCaptureId) == 0)
                {
                    MessageBox.Show("Error on chain_capture for tertiary capture");
                    this.CaptureButton.Enabled = false;
                }
                ChainTimer.Enabled = true;
            }
            else
            {
                this.VideoButton.Text = "Start";
                m_bCapturing = false;
                this.CaptureButton.Enabled = false;
                this.HistogramCheckBox.Checked = false;
                this.HistogramCheckBox.Enabled = false;
                if (m_CaptureId != 0)
                {
                    Deva.Camlib.reset_capture(m_CaptureId);
                    Deva.Camlib.free_capture(m_CaptureId);
                    m_CaptureId = 0;
                }
                if (m_StillCaptureId != 0)
                {
                    Deva.Camlib.reset_capture(m_StillCaptureId);
                    Deva.Camlib.free_capture(m_StillCaptureId);
                    m_StillCaptureId = 0;
                }
                if (m_ChainCaptureId != 0)
                {
                    Deva.Camlib.reset_capture(m_ChainCaptureId);
                    Deva.Camlib.free_capture(m_ChainCaptureId);
                    m_ChainCaptureId = 0;
                    ChainTimer.Enabled = false;
                }
            }
        }

        private void OnTimerTick(object sender, EventArgs e)
        {
            Deva.Camlib.CAPTURE_FRAME_INFO FrameInfo = new Deva.Camlib.CAPTURE_FRAME_INFO();
            if (Deva.Camlib.capture_occured(m_StillCaptureId, ref FrameInfo) == 1)
            {
                byte [] Buffer = new Byte [m_BuffSize];
                if (Deva.Camlib.copy_capture_buffer(m_StillCaptureId, ref Buffer,ref FrameInfo) == 1)
                {
                    if (m_nSkipCount > 0)       // skip a few frames so that the auto-setup adjustment has time to work
                    {
                        m_nSkipCount--;
                        // get another frame from the camera
                        Deva.Camlib.capture_control(m_StillCaptureId, Deva.Camlib.CAPTURE_CONTROL_TYPES.CAPTURE_CONTROL_TYPE_FRAME_ADVANCE);
                    }
                    else
                    {
                        // were done so display the data
                        Bitmap bmp = new Bitmap(2048, 1536);
                        // Specify a pixel format.
                        PixelFormat pxf = PixelFormat.Format32bppRgb;

                        // Lock the bitmap's bits.
                        Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                        BitmapData bmpData =
                        bmp.LockBits(rect, ImageLockMode.ReadWrite,
                                     pxf);

                        // Get the address of the first line.
                        IntPtr ptr = bmpData.Scan0;

                        // Declare an array to hold the bytes of the bitmap.
                        Int32 numBytes = Math.Abs(bmpData.Stride) * bmpData.Height;

                        // Copy the RGB values back to the bitmap
                        Marshal.Copy(Buffer, 0, ptr, numBytes);

                        // Unlock the bits.
                        bmp.UnlockBits(bmpData);

                        StillPictureForm[] NewPics = new StillPictureForm[m_nPictureIndex + 1];
                        for (int i = 0; i < m_nPictureIndex; i++)
                        {
                            NewPics[i] = m_Pics[i];
                        }
                        NewPics[m_nPictureIndex] = new StillPictureForm(bmp, ("Still Picture " + (m_nPictureIndex  / 2).ToString()), m_nPictureIndex);
                        NewPics[m_nPictureIndex].CloseEvent += new StillPictureForm.CloseEventHandler(OnPictureClosed);
                        NewPics[m_nPictureIndex].Show();
                        NewPics[m_nPictureIndex].Size = new Size(this.Width / 2, this.Height / 2);
                        m_nPictureIndex++;
                        m_Pics = NewPics;
                        // now the Histogram
                        bmp = new Bitmap(256,256);
                        // Specify a pixel format.
                        pxf = PixelFormat.Format32bppRgb;

                        // Lock the bitmap's bits.
                        rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                        bmpData =
                        bmp.LockBits(rect, ImageLockMode.ReadWrite,
                                     pxf);

                        // Get the address of the first line.
                        ptr = bmpData.Scan0;

                        // Declare an array to hold the bytes of the bitmap.
                        numBytes = Math.Abs(bmpData.Stride) * bmpData.Height;

                        Int32 [] rgbValues = new Int32[numBytes / sizeof(Int32)];

                        // Copy the RGB values into the array.
                        System.Runtime.InteropServices.Marshal.Copy(ptr, (Int32[])rgbValues,0,numBytes / sizeof(Int32));

                        int nRow;
                        int nCol;
                        for (nRow = 0; nRow < bmpData.Height; nRow++)
                        {
                            int nOffset = ((Math.Abs(bmpData.Stride) * nRow) / sizeof(Int32));
                            for (nCol = 0; nCol < bmpData.Width; nCol++)
                            {
                                rgbValues[nOffset  + nCol] = 0;
                            }
                        }
                        int nMax = 0;
                        for (int n = 0; n < 256; n++)
                        {
                            if (nMax < FrameInfo.redHist.bin[n])
                            {
                                nMax = FrameInfo.redHist.bin[n];
                            }
                            if (nMax < FrameInfo.greenHist.bin[n])
                            {
                                nMax = FrameInfo.greenHist.bin[n];
                            }
                            if (nMax < FrameInfo.blueHist.bin[n])
                            {
                                nMax = FrameInfo.blueHist.bin[n];
                            }
                        }
                        double dScale = 255.0 / nMax;
                        for (nCol = 0; nCol < bmpData.Width; nCol++)
                        {
                            int nOffset = ((Math.Abs(bmpData.Stride) * (255 - (int)(FrameInfo.redHist.bin[nCol] * dScale ))) / sizeof(Int32));
                            rgbValues[nOffset + nCol] |= 0x00ff0000;
                            nOffset = ((Math.Abs(bmpData.Stride) * (255 - (int)(FrameInfo.greenHist.bin[nCol] * dScale))) / sizeof(Int32));
                            rgbValues[nOffset + nCol] |= 0x0000ff00;
                            nOffset = ((Math.Abs(bmpData.Stride) * (255 - (int)(FrameInfo.blueHist.bin[nCol] * dScale))) / sizeof(Int32));
                            rgbValues[nOffset + nCol] |= 0x000000ff;
                        }
                        // Copy the RGB values back to the bitmap
                        System.Runtime.InteropServices.Marshal.Copy(rgbValues,0, ptr, numBytes / sizeof(Int32));
                        // Unlock the bits.
                        bmp.UnlockBits(bmpData);
                        NewPics = new StillPictureForm[m_nPictureIndex + 1];
                        for (int i = 0; i < m_nPictureIndex; i++)
                        {
                            NewPics[i] = m_Pics[i];
                        }
                        NewPics[m_nPictureIndex] = new StillPictureForm(bmp, ("Histogram " + (m_nPictureIndex / 2).ToString()), m_nPictureIndex);
                        NewPics[m_nPictureIndex].CloseEvent += new StillPictureForm.CloseEventHandler(OnPictureClosed);
                        NewPics[m_nPictureIndex].Show();
                        m_nPictureIndex++;
                        m_Pics = NewPics;
                        StillTimer.Enabled = false;     // re-enable still capture
                        this.CaptureButton.Enabled = true;
                        this.SaveImageButton.Enabled = true;
                        m_bCapturingStill = false;
                    }
                }
            }
        }

        private void OnChainTimer(object sender, EventArgs e)
        {
            Deva.Camlib.CAPTURE_FRAME_INFO FrameInfo = new Deva.Camlib.CAPTURE_FRAME_INFO();
            byte[] Buffer = new Byte[512 * 384 * 4];
            if (Deva.Camlib.capture_occured(m_ChainCaptureId,ref FrameInfo) == 1)
            {
                // to get the frame metrics we need to call copy_capture buffer even though we don't need the data
                Deva.Camlib.copy_capture_buffer(m_ChainCaptureId, ref Buffer, ref FrameInfo);
                FocusTextBox.Text = FrameInfo.focus.ToString("N3");
                FrameNumberTextBox.Text = FrameInfo.frame_number.ToString("N3");
                FrameQualityTextBox.Text = FrameInfo.frame_quality.ToString("N3");
                IntegrationTimeTextBox.Text = FrameInfo.paramset.integration_time.ToString("N3");
            }
        }

        private void OnLighting(object sender, EventArgs e)
        {
            LightingButton.Enabled = false;
            LightingForm LightForm = new LightingForm(LightingButton);
            LightForm.Show();
        }

        private void OnSaveImage(object sender, EventArgs e)
        {
            int nImageCount = 0;
            string [] ImageNames;
            int[] ImageIndices;
            for (int n = 0;n < m_nPictureIndex;n++)
            {
                if (m_Pics[n] != null)
                {
                    nImageCount++;
                }
            }
            if (nImageCount > 0)
            {
                ImageNames = new string[nImageCount];
                ImageIndices = new int[nImageCount];
                int ImageIndex = 0;
                int nSelectedIndex = -1;
                for (int n = 0; n < m_nPictureIndex; n++)
                {
                    if (m_Pics[n] != null)
                    {
                        if (n % 2 != 0)
                        {
                            ImageNames[ImageIndex] = ("Histogram " + (n / 2).ToString());
                            ImageIndices[ImageIndex] = n;
                            ImageIndex++;
                        }
                        else
                        {
                            ImageNames[ImageIndex] = ("Still Picture " + (n / 2).ToString());
                            ImageIndices[ImageIndex] = n;
                            ImageIndex++;
                        }
                    }
                }
                if (nImageCount > 1)
                {
                    ImageSelectForm SelectForm = new ImageSelectForm(ImageNames, nImageCount);
                    if (SelectForm.ShowDialog() == DialogResult.OK)
                    {
                        nSelectedIndex = ImageIndices[SelectForm.GetIndex()];
                    }
                    else
                    {
                        return;
                    }
                }
                else
                {
                    nSelectedIndex = ImageIndices[0];
                }
                SaveFileDialog FileDialog = new SaveFileDialog();
                FileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                FileDialog.Filter = "jpg files (*.jpg)|*.jpg|Bmp files (*.bmp)|*.bmp|Gif files (*.gif)|*.gif|Png files (*.png)|*.png";
                FileDialog.FilterIndex = 0;
                FileDialog.CheckFileExists = false;
                FileDialog.RestoreDirectory = true;
                if (nSelectedIndex % 2 != 0)
                {
                    FileDialog.Title = "Select File for Image : " + "Histogram " + (nSelectedIndex / 2).ToString();
                }
                else
                {
                    FileDialog.Title = "Select File for Image : " + "Still Picture " + (nSelectedIndex / 2).ToString();
                }
                if (FileDialog.ShowDialog() == DialogResult.OK)
                {       // we have selected a file
                    switch (FileDialog.FilterIndex)
                    {
                        case 1:
                            m_Pics[nSelectedIndex].GetImage().Save(FileDialog.FileName, ImageFormat.Jpeg);
                            break;

                        case 2:
                            m_Pics[nSelectedIndex].GetImage().Save(FileDialog.FileName, ImageFormat.Bmp);
                            break;

                        case 3:
                            m_Pics[nSelectedIndex].GetImage().Save(FileDialog.FileName, ImageFormat.Gif);
                            break;

                        case 4:
                            m_Pics[nSelectedIndex].GetImage().Save(FileDialog.FileName, ImageFormat.Png);
                            break;
                    }
                }
                else
                {
                    return;
                }
            }
        }

        public void OnPictureClosed(object sender, StillPictureFormEventArgs e)
        {
            m_Pics[e.nIndex()] = null;
            for (int i = 0; i < m_nPictureIndex; i++)
            {
                if (m_Pics[i] != null)
                {
                    return;
                }
            }
            this.SaveImageButton.Enabled = false;
        }

        private void OnHistogramChanged(object sender, EventArgs e)
        {
            if (HistogramCheckBox.Checked)
            {
                Deva.Camlib.capture_overlay(m_CaptureId, Deva.Camlib.CAPTURE_COORDINATE_TYPES.CAPTURE_COORDINATE_WINDOW,
                    Deva.Camlib.CAPTURE_OVERLAY_TYPES.CAPTURE_OVERLAY_ADJUSTED_HISTOGRAM, 0, 0, 0, 0);
            }
            else
            {
                Deva.Camlib.capture_overlay(m_CaptureId, Deva.Camlib.CAPTURE_COORDINATE_TYPES.CAPTURE_COORDINATE_WINDOW,
                    Deva.Camlib.CAPTURE_OVERLAY_TYPES.CAPTURE_OVERLAY_OFF, 0, 0, 0, 0);
            }
        }
    }
}