using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DevaCameraSample
{
    public partial class LightingForm : Form
    {
        private class GroupBoxWithoutBackgroundRepaint : GroupBox
        {
            public GroupBoxWithoutBackgroundRepaint()
            {
                this.SetStyle(ControlStyles.UserPaint, true);
                this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
                this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
                this.UpdateStyles();
            }
        }

        private class LED
        {
            private int _x = 0, _y = 0;
            private bool _on = false;
            private Brush _onColor = Brushes.White;

            /// <summary>
            /// Prevents a default instance of the <see cref="LED"/> class from being created.
            /// </summary>
            private LED() { }
            /// <summary>
            /// Initializes a new instance of the <see cref="LED"/> class.
            /// </summary>
            /// <param name="x">The x.</param>
            /// <param name="y">The y.</param>
            /// <param name="on">if set to <c>true</c> [on].</param>
            /// <param name="onColor">Color of the on.</param>
            /// <param name="offColor">Color of the off.</param>
            public LED(int x, int y, bool on, Brush onColor)
            {
                _x = x;
                _y = y;
                _on = on;
                _onColor = onColor;
            }
            /// <summary>
            /// Initializes a new instance of the <see cref="LED"/> class.
            /// </summary>
            /// <param name="x">The x.</param>
            /// <param name="y">The y.</param>
            /// <param name="on">if set to <c>true</c> [on].</param>
            /// <param name="onColor">Color of the on.</param>
            public LED(double x, double y, bool on, Brush onColor)
            {
                _x = (int)x;
                _y = (int)y;
                _on = on;
                _onColor = onColor;
            }

            /// <summary>
            /// Gets or sets a value indicating whether this <see cref="LED"/> is enable.
            /// </summary>
            /// <value>
            ///   <c>true</c> if enable; otherwise, <c>false</c>.
            /// </value>
            public bool EnableLED
            {
                get { return _on; }
                set { _on = value; }
            }

            /// <summary>
            /// Draws this instance to the specified Graphics object
            /// </summary>
            /// <param name="gfx">The GFX object.</param>
            public void Draw(Graphics gfx)
            {
                if (_on)
                {
                    gfx.FillEllipse(_onColor, _x, _y, 15, 15);
                    gfx.DrawEllipse(Pens.DarkGray, _x, _y, 15, 15);
                }
				else
				{
                    gfx.FillEllipse(Brushes.Gray, _x, _y, 15, 15);
					gfx.DrawEllipse(Pens.Gray, _x, _y, 15, 15);
				}
            }

            /// <summary>
            /// Gets or sets the on brush.
            /// </summary>
            /// <value>
            /// The on brush.
            /// </value>
            public Brush DrawColor
            {
                get
                {
                    return _onColor;
                }
                set
                {
                    _onColor = value;
                }
            }
        }
        private List<LED> m_Leds = new List<LED>();
        private int m_nNumUnits;
        private int m_currentLEDIntensity = 255;
        private int m_maxLEDIntensity = 255;
        private int m_currentDirection = 0;
        private int m_maxDirection = 16;
        private int m_currentWidth = 0;
        private int m_maxWidth = 16;
        private bool m_isPowerOn = false;
        private bool m_isMasterOn = false;

        private Image m_bufferImage = null;

        private bool m_InInit;
        private bool m_is022 = true;
        private bool m_is036 = false;
		private bool m_hasStepperControl = false;
        private System.Windows.Forms.Button m_LightButton;

        public LightingForm(System.Windows.Forms.Button LightButton)
        {
            m_LightButton = LightButton;
            m_InInit = true;
            InitializeComponent();
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.UpdateStyles();
            m_bufferImage = new Bitmap(this.Width, this.Height);
            using (Graphics gfx = Graphics.FromImage(m_bufferImage))
            {
                gfx.FillEllipse(Brushes.Gray, 20, 20, 220, 220);			//outer circle
                gfx.FillEllipse(SystemBrushes.Control, 80, 80, 100, 100);	//inner circle
            }
            int unitType = Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.UNIT_TYPE, 0);
            if (unitType == 1)
            {
                //we've got a 022
                m_is022 = true;
                //perform check for number of steppers
                int numSteppers = Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.NUM_STEPPERS, 0);
                if (numSteppers > 0)
                    this.m_hasStepperControl = true;
            }
            else if (unitType == 2)
            {
                //we'ev got a 026
                this.m_is022 = false;
                this.m_hasStepperControl = true;
            }
            else if (unitType == 3)
            {
                //we'ev got a 036
                this.m_is022 = false;
                this.m_is036 = true;
                this.m_hasStepperControl = false;
            }
            if (this.m_is022)
            {
                BuildRingControl(64);
            }
            else
            {
                if (this.m_is036)
                {
                    //we'll show the aux2 panel + 'control' panel
                    BuildRingControl(Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.RING_WIDTH_MAX, 0));

                }
                else
                {
                    BuildRingControl(16);
                }
            }

            if (this.m_hasStepperControl)
            {
            }
            else
            {
            }
            m_InInit = false;
            m_nNumUnits = (int)Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.NUM_UNITS, 0);
            m_isMasterOn = (Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.ENABLE, 0) == 0 ? false : true);
            m_isPowerOn = (Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.RING_ENABLE, 0) == 0 ? false : true);
            MasterEnable = m_isMasterOn;
            RingEnable = m_isPowerOn;
            RingDirection = (int)(Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.RING_DIRECTION, 0));
            RingPower = (int)(Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.RING_POWER, 0));
            RingWidth = (int)(Deva.Lghtlib.read_light((short)Deva.Lghtlib.COMMANDS.RING_WIDTH, 0));
        }

        /// <summary>
        /// Builds the ring control.
        /// </summary>
        /// <param name="isExtendedLightRing">if set to <c>true</c> [is extended light ring].</param>
        public void BuildRingControl(int numLEDs)
        {
            switch (numLEDs)
            {
                case 64:
                    m_maxWidth = 32;
                    m_currentWidth = 0;
                    m_maxLEDIntensity = 255;
                    m_currentLEDIntensity = 127;
                    m_maxDirection = 31;
                    m_currentDirection = 0;
                    m_isPowerOn = false;
                    m_isMasterOn = false;
                    BuildExtendedLightring();
                    break;
                case 32:
                    m_maxWidth = 32;
                    m_currentWidth = 0;
                    m_maxLEDIntensity = 255;
                    m_currentLEDIntensity = 127;
                    m_maxDirection = 31;
                    m_currentDirection = 0;
                    m_isPowerOn = false;
                    m_isMasterOn = false;
                    Build036Lightring();
                    break;
                case 16:
                    m_maxWidth = 16;
                    m_currentWidth = 0;
                    m_maxLEDIntensity = 255;
                    m_currentLEDIntensity = 127;
                    m_maxDirection = 15;
                    m_currentDirection = 0;
                    m_isPowerOn = false;
                    m_isMasterOn = false;
                    BuildBasicLightring();
                    break;
            }

            //update width slider & settings
            int tempWidthVal = this.WidthVScrollBar.Value;
            this.WidthVScrollBar.Value = this.WidthVScrollBar.Maximum;
            this.WidthVScrollBar.Minimum = m_maxWidth * -1;
            if (tempWidthVal >= this.WidthVScrollBar.Minimum)
                this.WidthVScrollBar.Value = tempWidthVal;

            //update direction slider & settings
            int tempDirVal = this.DirectionHScrollBar.Value;
            this.DirectionHScrollBar.Value = DirectionHScrollBar.Minimum;
            this.DirectionHScrollBar.Maximum = m_maxDirection;
            if (tempDirVal <= this.DirectionHScrollBar.Maximum)
                this.DirectionHScrollBar.Value = tempDirVal;

            this.PowerVScrollBar.Minimum = -255;
        }
        /// <summary>
        /// Gets or sets a value indicating whether [master enable].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [master enable]; otherwise, <c>false</c>.
        /// </value>
        public bool MasterEnable
        {
            get
            {
                return m_isMasterOn;
            }
            set
            {
                m_isMasterOn = value;
                UpdateLEDsForNewSettings();
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="LightRing_Control"/> is enable.
        /// </summary>
        /// <value>
        ///   <c>true</c> if enable; otherwise, <c>false</c>.
        /// </value>
        public bool RingEnable
        {
            get
            {
                return m_isPowerOn;
            }
            set
            {
                m_isPowerOn = value;
                EnableCheckBox.Checked = (m_isPowerOn && m_isMasterOn);
                UpdateLEDsForNewSettings();
            }
        }

        /// <summary>
        /// Gets or sets the intensity.
        /// </summary>
        /// <value>
        /// The intensity.
        /// </value>
        public int RingPower
        {
            get
            {
                return m_currentLEDIntensity;
            }
            set
            {
                int tmp = value;

                if (tmp > m_maxLEDIntensity)
                    tmp = m_maxLEDIntensity;

                this.m_currentLEDIntensity = tmp;

                this.PowerVScrollBar.Value = (-1 * tmp);
                this.PowerTextBox.Text = tmp.ToString();
                UpdateLEDsForNewSettings();
            }
        }

        /// <summary>
        /// Gets or sets the direction.
        /// </summary>
        /// <value>
        /// The direction.
        /// </value>
        public int RingDirection
        {
            get
            {
                return m_currentDirection;
            }
            set
            {
                int tmp = value;
                if (tmp > this.m_maxDirection)
                    tmp = m_maxDirection;

                m_currentDirection = tmp;

                this.DirectionHScrollBar.Value = tmp;
                this.DirectionTextBox.Text = tmp.ToString();
                UpdateLEDsForNewSettings();
            }
        }

        /// <summary>
        /// Gets or sets the width of the control.
        /// </summary>
        /// <returns>
        /// The width of the control in pixels.
        ///   </returns>
        public int RingWidth
        {
            get
            {
                return m_currentWidth;
            }
            set
            {
                int tmp = value;
                if (tmp > this.m_maxWidth)
                    tmp = this.m_maxWidth;

                this.m_currentWidth = tmp;

                this.WidthVScrollBar.Value = (-1 * tmp);
                this.WidthTextBox.Text = tmp.ToString();
                UpdateLEDsForNewSettings();
            }
        }

        /// <summary>
        /// Builds the basic lightring.
        /// </summary>
        private void BuildBasicLightring()
        {
            //new lightring count = 16LEDs
            if (m_Leds.Count != 16)
            {
                //clear the current led list
                m_Leds.Clear();
                //calculate new draw arrangement

                int rad = 80;
                int toppos = 122;
                int leftpos = 123;
                for (int index = 0; index < 16; index++)
                {
                    m_Leds.Add(new LED(leftpos + rad * Math.Cos((15.5 - index) / 8 * Math.PI),
                                        toppos - rad * Math.Sin((15.5 - index) / 8 * Math.PI),
                                        m_isPowerOn, GetBrushFromIntensity()));
                }
            }
            this.LightRingGroupBox.Invalidate();
        }

        /// <summary>
        /// Builds the extended lightring.
        /// </summary>
        private void BuildExtendedLightring()
        {
            //new lightring count = 64 LEDs
            if (m_Leds.Count != 64)
            {
                //clear the current led list
                m_Leds.Clear();
                //calculate new draw arrangement

                int innerRadius = 65;
                int innerAltRadius = 85;
                int outerRadius = 75;
                int outerAltRadius = 95;

                int toppos = 122;
                int leftpos = 123;
                LED[] ledArray = new LED[64];

                for (int index = 0; index < 64; index += 4)
                {
                    ledArray[index] = new LED(leftpos + innerRadius * Math.Cos((index) / 32d * Math.PI),
                                                toppos - innerRadius * Math.Sin((index) / 32d * Math.PI),
                                                m_isPowerOn, GetBrushFromIntensity());
                    //}
                    //radius = 85;
                    //for(int index=0; index < 64; index+=4)
                    //{
                    ledArray[index + 1] = new LED(leftpos + innerAltRadius * Math.Cos((index) / 32d * Math.PI),
                                                    toppos - innerAltRadius * Math.Sin((index) / 32d * Math.PI),
                                                    m_isPowerOn, GetBrushFromIntensity());
                    //}
                    //radius = 75;
                    //for(int index=0; index < 64; index+=4)
                    //{
                    ledArray[index + 2] = new LED(leftpos + outerRadius * Math.Cos((index + 2) / 32d * Math.PI),
                                                    toppos - outerRadius * Math.Sin((index + 2) / 32d * Math.PI),
                                                    m_isPowerOn, GetBrushFromIntensity());
                    //}
                    //radius = 95;
                    //for(int index=0; index < 64; index+=4)
                    //{
                    ledArray[index + 3] = new LED(leftpos + outerAltRadius * Math.Cos((index + 2) / 32d * Math.PI),
                                                    toppos - outerAltRadius * Math.Sin((index + 2) / 32d * Math.PI),
                                                    m_isPowerOn, GetBrushFromIntensity());
                }
                this.m_Leds.AddRange(ledArray);

                //force this control to re-draw at the next cycle
                this.LightRingGroupBox.Invalidate();
            }
        }

        private void Build036Lightring()
        {
            //new lightring count = 32 LEDs
            if (m_Leds.Count != 32)
            {
                //clear the current led list
                m_Leds.Clear();
                //calculate new draw arrangement

                int innerRadius = 75;
                int outerRadius = 90;

                int toppos = 122;
                int leftpos = 123;
                LED[] ledArray = new LED[32];

                for (int index = 0; index < 32; index += 2)
                {
                    ledArray[index] = new LED(leftpos + innerRadius * Math.Cos((index) / 16d * Math.PI),
                                                toppos - innerRadius * Math.Sin((index) / 16d * Math.PI),
                                                m_isPowerOn, GetBrushFromIntensity());

                    ledArray[index + 1] = new LED(leftpos + outerRadius * Math.Cos((index + 1) / 16d * Math.PI),
                                                    toppos - outerRadius * Math.Sin((index + 1) / 16d * Math.PI),
                                                    m_isPowerOn, GetBrushFromIntensity());
                }
                this.m_Leds.AddRange(ledArray);

                //force this control to re-draw at the next cycle
                this.LightRingGroupBox.Invalidate();
            }
        }

        /// <summary>
        /// Updates the LEDs for new settings.
        /// </summary>
        private void UpdateLEDsForNewSettings()
        {
            Brush color = GetBrushFromIntensity();
            int modulusValue = 0;
            int startIndex = 0;

            if (m_InInit)
            {
                return;
            }

            switch (this.m_Leds.Count)
            {
                case 64:
                    modulusValue = 32;
                    startIndex = m_currentDirection - ((m_currentWidth - 1) / 2);
                    break;
                case 32:
                    modulusValue = 32;
                    startIndex = m_currentDirection - ((m_currentWidth - 1) / 2);
                    break;
                case 16:
                    modulusValue = 16;
                    startIndex = -m_currentDirection - ((m_currentWidth - 1) / 2);
                    break;
            }

            int endIndex = startIndex + m_currentWidth - 1;

            startIndex = (startIndex + 32) % modulusValue;
            endIndex = (endIndex + 32) % modulusValue;

            for (int index = 0; index < modulusValue; index++)
            {
                if (((endIndex >= startIndex && index >= startIndex && index <= endIndex) ||
                    (startIndex > endIndex && (index >= startIndex || index <= endIndex))) &&
                    m_currentWidth > 0)
                {
                    switch (this.m_Leds.Count)
                    {
                        case 64:
                            int extendedIndex = index * 2;
                            this.m_Leds[extendedIndex].DrawColor = color;
                            this.m_Leds[extendedIndex].EnableLED = this.m_isPowerOn && this.m_isMasterOn;
                            this.m_Leds[extendedIndex + 1].DrawColor = color;
                            this.m_Leds[extendedIndex + 1].EnableLED = this.m_isPowerOn && this.m_isMasterOn;
                            break;
                        case 32:
                            this.m_Leds[index].DrawColor = color;
                            this.m_Leds[index].EnableLED = this.m_isPowerOn && this.m_isMasterOn;
                            break;
                        case 16:
                            this.m_Leds[index].DrawColor = color;
                            this.m_Leds[index].EnableLED = this.m_isPowerOn && this.m_isMasterOn;
                            break;
                    }
                }
                else
                {
                    switch (this.m_Leds.Count)
                    {
                        case 64:
                            int extendedIndex = index * 2;
                            this.m_Leds[extendedIndex].EnableLED = false;
                            this.m_Leds[extendedIndex + 1].EnableLED = false;
                            break;
                        case 32:
                            this.m_Leds[index].EnableLED = false;
                            break;
                        case 16:
                            this.m_Leds[index].EnableLED = false;
                            break;
                    }
                }
            }

            using (Graphics gfx = Graphics.FromImage(m_bufferImage))
            {
                foreach (LED led in m_Leds)
                {
                    led.Draw(gfx);
                }
            }
            this.LightRingGroupBox.Invalidate(new Rectangle(20, 20, 220, 220),false);	//invalidate only the lightring section of the image
        }

        /// <summary>
        /// Gets the brush from intensity.
        /// </summary>
        /// <returns></returns>
        private Brush GetBrushFromIntensity()
        {
            int drawVal = (this.m_currentLEDIntensity / 2) + 127;
            return new SolidBrush(Color.FromArgb(drawVal,
                                                drawVal,
                                                drawVal));
        }

        private void OnClose(object sender, EventArgs e)
        {
            m_LightButton.Enabled = true;
            this.Close();
        }

        private void EnableChanged(object sender, EventArgs e)
        {
            this.m_isPowerOn = this.EnableCheckBox.Checked;
            this.m_isMasterOn = this.m_isPowerOn;
            // update the value on the lightring here
            if (!m_InInit)
            {
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.ENABLE, 0, this.m_isMasterOn != false ? 1 : 0);
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.RING_ENABLE, 0, this.m_isPowerOn != false ? 1 : 0);
                UpdateLEDsForNewSettings();
            }
        }

        private void OnDirectionScroll(object sender, ScrollEventArgs e)
        {
            this.m_currentDirection = this.DirectionHScrollBar.Value;
            this.DirectionTextBox.Text = this.DirectionHScrollBar.Value.ToString();
            if (!m_InInit)
            {
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.RING_DIRECTION, 0, this.m_currentDirection);
                UpdateLEDsForNewSettings();
            }
        }

        private void OnWidthScroll(object sender, ScrollEventArgs e)
        {
            this.m_currentWidth = (-1 * this.WidthVScrollBar.Value);
            this.WidthTextBox.Text = (-1 * this.WidthVScrollBar.Value).ToString();
            if (!m_InInit)
            {
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.RING_WIDTH, 0, this.m_currentWidth);
                UpdateLEDsForNewSettings();
            }
        }

        private void OnPowerScroll(object sender, ScrollEventArgs e)
        {
            this.m_currentLEDIntensity = -1 * this.PowerVScrollBar.Value;
            this.PowerTextBox.Text = (-1 * this.PowerVScrollBar.Value).ToString();
            if (!m_InInit)
            {
                Deva.Lghtlib.write_light((short)Deva.Lghtlib.COMMANDS.RING_POWER, 0, this.m_currentLEDIntensity);
                UpdateLEDsForNewSettings();
            }
        }

        private void OnLightRingGroupBoxPaint(object sender, PaintEventArgs e)
        {
            //local double buffering attempt
            if (m_bufferImage != null)
            {
                e.Graphics.DrawImage(m_bufferImage, e.ClipRectangle.X, e.ClipRectangle.Y, e.ClipRectangle, System.Drawing.GraphicsUnit.Pixel);
            }
        }

        private void OnClosing(object sender, FormClosingEventArgs e)
        {
            m_LightButton.Enabled = true;
        }
    }
}