/*----------------------------------------------------------------------------

	USB device driver header file

--------------------------------------------------------------------------

	Revision History.

----------------------------------------------------------------------------*/

#ifdef _WIN32

#define PTR_DEF
#define FN_DEF __stdcall

#endif

#ifdef __cplusplus

#define EXT_DEF extern "C"

#else

#define EXT_DEF

#endif	

#define MAX_COLUMNS 2048
#define MAX_ROWS 1536
#define MAX_CAMERA_DESCRIPTION 32

//#define WORDWIDE_ENABLE

#ifdef WORDWIDE_ENABLE
#define MAX_FRAME_SIZE (2048L*1536L*2)
#else
#define MAX_FRAME_SIZE (2048L*1536L)
#endif

#define MAX_CAMERAS 4
#define MAX_CAPTURE_REQUIREMENTS_PER_CAMERA 4
#define MAX_CAPTURE_SYNCS_PER_CAMERA 4
#define TOTAL_CAPTURE_REQUIREMENTS (MAX_CAPTURE_REQUIREMENTS_PER_CAMERA * MAX_CAMERAS)
#define MAX_CAPTURE_FILTER_QUEUE_LEN 100
#define FRAME_STATE_TIMEOUT 2000

enum CAM_CAPTURE_TYPES
{
	CAPTURE_STOP=0,
	CAPTURE_IMMEDIATE=1,
	CAPTURE_ON_PROBE=2,
	CAPTURE_REPEAT=3
};

enum CAM_OPERATION_MODES
{
	CAM_STANDARD_RES_FULL_SPEED=0,
	CAM_HIGH_RES_HALF_SPEED=1
};
enum CAM_SYNC_TYPES
{
	CAM_SYNC_OFF=0,
	CAM_SYNC_CAPTURE_PULSE=1,
	CAM_SYNC_CAPTURE_SET=2,
	CAM_SYNC_PULSE=3,
	CAM_SYNC_PULSE_WAIT_OFF=99
};

enum CAPTURE_CONTROL_TYPES
{
	CAPTURE_CONTROL_TYPE_STOP=0,
	CAPTURE_CONTROL_TYPE_PAUSE=1,
	CAPTURE_CONTROL_TYPE_PLAY=2,
	CAPTURE_CONTROL_TYPE_FRAME_ADVANCE=3
};

enum RD_WR_COMMANDS
{
	USB_CAMERA_REGISTER=0,
	USB_CAMERA_EEPROM=1,
	USB_CAPTURE_SETUP=2,
	USB_CAPTURE_CONTROL=3,
	USB_CAPTURE_ALLOC_REQUIREMENT=4,
	USB_CAPTURE_DATA_POINTER_USER=5,
	USB_CAPTURE_DATA_POINTER_WDM=6,
	USB_CAPTURE_RESET=7,
	USB_CAMERA_RESET=8,
	USB_SYNC_SETUP=9,
	USB_USER_CAPTURE_EVENT=10,
	USB_KERNEL_CAPTURE_EVENT=11,
	USB_SETUP_CAPTURE_CHAIN=12,
	USB_CAPTURE_CHAIN=13,
	USB_CAPTURE_REQ_POINTER_USER=15,
	USB_CAPTURE_REQ_POINTER_WDM=16,
	USB_CAMERA_OPERATING_MODE=17,
	USB_CAMERA_INFO=18,
	USB_CAMERA_RESYNC_REQUEST=19,
	USB_CAMERA_FREE_ALL_CAPTURES=20,
	USB_CAMERA_SETTINGS=21,
	USB_CAPTURE_USER_DATA_POINTER_USER=25,
	USB_CAPTURE_USER_DATA_POINTER_WDM=26,
	USB_CAMERA_TEST_STRING = 30
};

typedef struct
{
	unsigned short num;
	unsigned short time_stamp;
	unsigned long total_frame_time;

	unsigned short capture_index;
	unsigned short capture_type;
	unsigned short cam_format;
	unsigned short cam_mode;

	short cam_cx;
	short cam_cy;
	short cam_dx;
	short cam_dy;
	
	short res;
	short repeat_interval;
	short gain;
	short crc;

	long integration_time;
	long frame_size;

	unsigned char tl1;
	unsigned char th1;
	unsigned short spare1;

} CAPTURE_FRAME_HEADER;

typedef struct
{
	long missed_pixels;
	unsigned short crc;
	unsigned short commit_frame;
	unsigned char sync_valid;
	unsigned char pad1;
	unsigned short pad2;
} CAPTURE_FRAME_TERMINATOR;


typedef struct
{
	unsigned short sync_type;
	unsigned short invert_logic;
	unsigned short sync_index;
	unsigned short capture_index;
} CAPTURE_SYNC_DATA;

typedef struct {
	double item[MAX_CAPTURE_FILTER_QUEUE_LEN];
	double value;
	unsigned short head;
	unsigned short tail;
	unsigned short len;
} CAPTURE_FILTER_QUEUE;

/*----------------------------------------------------------------------------

	Local function prototypes

----------------------------------------------------------------------------*/
unsigned long CamOpenIO(void);
void CamCloseIO(void);

unsigned long dlread_usb_file_index_direct_io(unsigned long file_index, long command, unsigned char* pIOData, long size, long *retval);
unsigned long dlread_usb_file_index(unsigned long file_index, long command, unsigned char* pReadData, long size, long *retval);
unsigned long dlwrite_usb_file_index(unsigned long file_index, long command, unsigned char* pWriteData, long size, long *retval);

